// cbi and sbi are standard (AVR) methods for setting, 
// or clearing, bits in PORT (and other) variables.
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif 

//---------------------------------------------------------------------- 
void EnableADC()
  { // This is probably not needed but ....
    // set system clock devisor to 128
    // 16 MHz / 128 = 125 KHz, inside the desired 50-200 KHz range.
    sbi(ADCSRA, ADPS2);                            // bit 2 of ADCSRA, system clock devisor
    sbi(ADCSRA, ADPS1);                            // bit 1 of ADCSRA, system clock devisor
    sbi(ADCSRA, ADPS0);                            // bit 0 of ADCSRA, system clock devisor
    cbi(ADCSRA, ADATE);                            // bit 5 of ADCSRA, disable auto trigger mode
    sbi(ADCSRA, ADEN);                             // bit 7 of ADCSRA, enable ADC 
  } 

//---------------------------------------------------------------------- 
void Read_Calibration_Data()
 { byte i=0, j=0;
   word  TempWord;
   byte  TempByte;
   char c;
   char sign;
   
   // detect a virgin device --- well at least try
   // This is the one entry that should not be FFh
   if ((EEPROM.read(EEminutes)==0xFF) &&
       (EEPROM.read(EEminutes+1)==0xFF))
	   TestData1();

   // Get Valid sensors
   for (i=0; i<8;i++) Sensors[i]=false;
   TempByte= EEPROM.read(EEsensors);
   if ((TempByte & B00000001)!=0)Sensors[0]=true;
   if ((TempByte & B00000010)!=0)Sensors[1]=true;
   if ((TempByte & B00000100)!=0)Sensors[2]=true;
   if ((TempByte & B00001000)!=0)Sensors[3]=true;
   if ((TempByte & B00010000)!=0)Sensors[4]=true;
   if ((TempByte & B00100000)!=0)Sensors[5]=true;
   if ((TempByte & B01000000)!=0)Sensors[6]=true;
   if ((TempByte & B10000000)!=0)Sensors[7]=true;
   
   // Get Operational modes
   TempByte= EEPROM.read(EEopmodes);
   ReportMode=false;
   RtnRawRead=false;
   RtnCelsius=false;
   RtnFahrenh=false;
   DeBug=false;
   RoundMode=false;
   EepromMode=false;
   if ((TempByte & B00000001)!=0)ReportMode=true;
   if ((TempByte & B00000010)!=0)RtnRawRead=true;
   if ((TempByte & B00000100)!=0)RtnCelsius=true;
   if ((TempByte & B00001000)!=0)RtnFahrenh=true;
   if ((TempByte & B00010000)!=0)DeBug=true;
   if ((TempByte & B00100000)!=0)RoundMode=true;
   //if ((TempByte & B01000000)!=0)EepromMode=true;
   //if ((TempByte & B10000000)!=0)???=true;
 
   // Get Degree Offset
   TempWord= EEPROM.read(EEoffset)<<8;
   TempWord= (TempWord + EEPROM.read(EEoffset +1));
   sign= 1;
   if ((TempWord>>12) == B1000) sign= -1;
   // strip the sign bit
   TempWord=(TempWord<<1)>>1;
   // now we need to convert it to a fraction
   DegreeOffset=(float(TempWord)/float(CovrtFactorO)) * sign;

   // Get the RefVoltage factor;
   TempWord= EEPROM.read(EErefvolt)<<8;
   TempWord= TempWord + EEPROM.read(EErefvolt +1);
   // now we need to convert it to a fraction
   RefVoltage = float(TempWord)/CovrtFactorV ;

   // ID String -------------------------------------
   i=0; c=1;
   while (c!=0,i< EEidsize)
    {c=EEPROM.read(EEidtring + i);         // read the ID string
     IdString [i++]=c; 
    } 
   IdString [EEidsize]=0;                  // just in case

   // Set MinuteTarget from default minutes
   MinuteTarget=(EEPROM.read(EEminutes)<<8) + EEPROM.read(EEminutes +1);
   if (MinuteTarget<1)MinuteTarget=1;
   
   // this is a bit flag to indicate when the current constants
   // in memory are different from those stored in working storage 
   newflg=0; 
   
 }
 
//----------------------------------------------------------------------  
void Write_Calibration_Data()
  { word i=0;
    word  TempWord;
    byte  TempByte;
    char c=-1;
    byte sign;

    // Sensor Status
    if (true) 
      { TempByte=0;
        if (Sensors[0]) TempByte=TempByte | B00000001;
        if (Sensors[1]) TempByte=TempByte | B00000010;
        if (Sensors[2]) TempByte=TempByte | B00000100;
        if (Sensors[3]) TempByte=TempByte | B00001000;
        if (Sensors[4]) TempByte=TempByte | B00010000;
        if (Sensors[5]) TempByte=TempByte | B00100000;
        if (Sensors[6]) TempByte=TempByte | B01000000;
        if (Sensors[7]) TempByte=TempByte | B10000000;
        if (EEPROM.read(EEsensors)!=TempByte) EEPROM.write(EEsensors,TempByte);
      } 
    
    // Operational modes
    if (true) 
      { TempByte=0;
        if (ReportMode) TempByte=TempByte | B00000001;
        if (RtnRawRead) TempByte=TempByte | B00000010;
        if (RtnCelsius) TempByte=TempByte | B00000100;
        if (RtnFahrenh) TempByte=TempByte | B00001000;
        if (DeBug)      TempByte=TempByte | B00010000;
        if (RoundMode)  TempByte=TempByte | B00100000;
        if (EepromMode) TempByte=TempByte | B01000000;   // for compatibility only
        // if (Sensors[7]) TempByte=TempByte | B10000000;
        if (EEPROM.read(EEopmodes)!=TempByte) EEPROM.write(EEopmodes,TempByte);
      } 
    
    
    // New location
    if ((newflg & B00000001) == B00000001)
      {
        for (i=0;i<EEidsize; i++) {EEPROM.write((EEidtring +i), IdString[i]);}
      }

    // New Degree Offset
    if ((newflg & B00000010) == B00000010)
      { sign = 0;
        if (DegreeOffset <0) sign = B10000000;
        TempWord=word(abs(DegreeOffset) * CovrtFactorO);   
        EEPROM.write(EEoffset   , highByte(TempWord)|sign);
        EEPROM.write(EEoffset +1, lowByte (TempWord));
      }
    // New Reference Voltage
    if ((newflg & B00000100) == B00000100)
      { TempWord = word(RefVoltage * CovrtFactorV);
        EEPROM.write(EErefvolt   , highByte(TempWord));
        EEPROM.write(EErefvolt +1, lowByte (TempWord));
      }
      
    // New MinuteTarget
    if ((newflg & B00010000) == B00010000)
      {
        EEPROM.write(EEminutes   , highByte(MinuteTarget));
        EEPROM.write(EEminutes +1, lowByte (MinuteTarget));
      }

   // note newflg is reset by Read_Calibration_Data
   Read_Calibration_Data();
   // PrintOKStr(); is sent by Report_Reset
   Report_Reset();
  } 
 
//----------------------------------------------------------------------  

void Print_IdString()
 { // Serial.print(F("; "));                    // pefix 
   Serial.println(IdString);                 // print it
 }
 
//-----------------------------------------------------
void PrintTrueFalse(byte T) 
 { // used to report True or False for boolean Globals
   if(T == 0) Serial.println(F("False"));
   else  Serial.println(F("True"));
 }

//-----------------------------------------------------
void PrintSensors() 
 { // use to print Valid Sensors
 // prints a dot for each sensor unless the sensor is true
 // if the sensor is valid prints the number for the sensor
   char string[9];
   string[8]=0;
   for (byte i; i<8;i++)
     { if (Sensors[i]) string[i]='0'+i;
       else string[i]='.';
     }  
   Serial.println(string);
 }
 
     
//-----------------------------------------------------
void ReportStatus()
 { // report settings
   PrintSeperatorLine();
   Serial.print   (F(";    Report:\t"));
   PrintTrueFalse (ReportMode);
   Serial.print   (F(";     Debug:\t"));
   PrintTrueFalse (DeBug);
   Serial.print   (F(";       Raw:\t"));
   PrintTrueFalse (RtnRawRead);
   Serial.print   (F(";Fahrenheit:\t"));
   PrintTrueFalse (RtnFahrenh);
   Serial.print   (F(";   Celsius:\t"));
   PrintTrueFalse (RtnCelsius);
   Serial.print   (F(";     Round:\t"));
   PrintTrueFalse (RoundMode);
   
   Serial.print   (F(";   Minutes:\t"));
   Serial.println (MinuteTarget, DEC);
   Serial.print   (F(";   Voltage:\t"));
   Serial.println (RefVoltage, 4);
   
  
   Serial.print   (F("; Sensor ID:\t"));
   Print_IdString ();
   Serial.print   (F(";    Offset:\t"));
   Serial.println (DegreeOffset, 4);
   
   Serial.print   (F(";   Sensors:\t"));
   PrintSensors();

   if (newflg != 0) 
     Serial.println (F(";Parameters not saved"));
   
   PrintSeperatorLine();
   
 }
//-----------------------------------------------------------------------------------------
//    ---- Below is the function the read the External LM34DZ Temperature Sensor ----
//-----------------------------------------------------------------------------------------
void ReadRawTemp()
  { // Read ADC for Pin A1 (connected to LM34DZ Temperature Sensor)
    // Cycle time is aproximately 124 miliseconds
    unsigned long RawSum;                         // used to sum samples for averaging 
    word Garbage;                                 // used to accumalate 10 bit ADC readings 
    word Count;                                   // used to count samples
    for (byte i=0; i<8; i++)
      { if (Sensors[i])
          { 
          RawSum=0;
          Count=0;
          pinMode(PinNo[i],INPUT);
          analogReference(INTERNAL);
          delay(10);
          Garbage=analogRead(PinNo[i]);          // Toss first reading
          delay(5);
          while (Count++ < 512 ) RawSum = RawSum + analogRead(PinNo[i]);
          Accumalator[i]= Accumalator[i]+(RawSum/512);
          }
      }
    CycleCount++;                                   // used by functions to average readings 
  }

//-----------------------------------------------------------------------------------------
void Convert(word RawReading)
  { // converts Raw Reading to Celsius and Fahrenheit
    // New plan: offset will only be used for minor correction
    // SCALE is actual voltage that is supposed to be 1.1 but reads 1.067
    // This temperature sensor reports in Fahrenheit 1 milivolt per degree
    // We need the correct voltage !!!
    // Covrt2Fahrenheit=1.067;
    Fahrenheit = (float(RawReading)/1024)* RefVoltage *100;
    Fahrenheit = Fahrenheit + DegreeOffset;
    if (RoundMode) Fahrenheit= nearesthalf(Fahrenheit);
    // Consistancy is next to godliness.
    // We are working in Fahrenheit. 
    // In our case Celsius is a function of Fahrenheit.
    // Thus we always complete our Fahrenheit cals first.
    // That inludes rounding.
    Celsius   = (Fahrenheit-32)/1.8;
    if (RoundMode) Celsius=nearestquater(Celsius);
  }

//-----------------------------------------------------
float nearestquater (float ValueIn)
  { // Return value rounded to nearest quater (0.25)
    byte sign=1;
    float fraction;
    if (ValueIn , 0) 
      {
      sign = -1; 
      ValueIn=abs(ValueIn);
      }
    fraction =ValueIn-long(ValueIn);
    if      (fraction >= 0.875) fraction=1.00; 
    else if (fraction >= 0.625) fraction=0.75;
    else if (fraction >= 0.375) fraction=0.50;
    else if (fraction >= 0.125) fraction=0.25;
    else                        fraction=0;
    return (long(ValueIn)+fraction) * sign;
  }
//-----------------------------------------------------
float nearesthalf (float ValueIn)
  { // Return value rounded to nearest half (0.50)
    byte sign=1;
    float fraction;
    if (ValueIn , 0) 
      {
      sign = -1; 
      ValueIn=abs(ValueIn);
      }
    fraction =ValueIn-long(ValueIn);
    if      (fraction >= 0.750) fraction=1.00; 
    else if (fraction >= 0.250) fraction=0.50;
    else                        fraction=0;
    return (long(ValueIn)+fraction) * sign;
  }
//-----------------------------------------------------
void Report()
 { word AvgSumRead;
   char ID[5];
   ID[4]=0;
   ID[3]='\t';
   ID[2]=':';
   ID[0]='A';
   for (byte i=0; i<8; i++)
     { // handle each sensor one at a time
       if (Sensors[i])
         {  AvgSumRead = Accumalator[i]/CycleCount;
            ID[1]='0'+i;
            Serial.print  (ID);
            Convert (AvgSumRead);
            if (RtnRawRead)
              { Serial.print  (AvgSumRead);
                Serial.print  (char(9));
              }
            if (RtnCelsius)
              { Serial.print  (Celsius,2);
                Serial.print  (char(9));
              }
            if (RtnFahrenh)
              { Serial.print  (Fahrenheit,2);
                Serial.print  (char(9));
              } 
         }
     }
     
   Serial.println();
   if (DeBug)
      { // move debug to serperate line with semicolon
        Serial.print (F("; Debug Info:\t"));
        Serial.print  (CycleTime/CycleCount);
        Serial.print  (char(9));
        Serial.print  (CycleCount);
        Serial.print  (char(9));
        Serial.print  (millis()-RptStartTime);
        Serial.println();
      }
  RptStartTime=millis();
  for (byte i=0; i<8; i++) Accumalator[i] = 0;    
  CycleTime   = 0;
  CycleCount  = 0;
   
 }

//------------------------------------------------------
void QuickBlink()
 { // on the UNO 1 mullisecond will surfice
   // adjusted up to 3 for Nano
   digitalWrite(LED, HIGH);     // turn on LED
   delay(3);       
   digitalWrite(LED, LOW);     // turn off LED
 }

//---------------------------------------------------------------------- 
void Responce (char str[])
  {if (EepromMode == false)
     { // we do not want to get hung up 
       // this just serves to reduce command responce memory usage a bit
       // trying to wrtie to seomthing that is not connected
       Serial.print (F("; "));
       Serial.print (cmd); 
       Serial.print (F(" "));
       Serial.println (str);
     }
  } 
//---------------------------------------------------------------------- 
void PrintOKStr ()
 { // command was accepted and processed
   Responce ("OK");
 }  

//---------------------------------------------------------------------- 
void PrintNotRecognized()
 { // command was Not Recognized
   Responce ("??");
 }  

//---------------------------------------------------------------------- 
void PrintNotImplemented()
 { // command was Not Recognized
   Responce ("XX");
 }  

//-----------------------------------------------------
void ShutDown()
  { // Note that no provision is made to wake up.
    // This is as close to shutdown as we can get.
    // Because of the inefficent voltage regulator this 
    // mode still draws a lot of power (about 10mA). 
    // A standard 9 volt battery may last about 16 hours.

    // Serial.println(prevcmd);
    if ((prevcmd[0]=='S') && (prevcmd[1]=='S'))
      { Serial.println (F("; SHUTDOWN"));
        // give device time to send string
        for (byte i=0; i< 25; i++)
          { QuickBlink();
            delay (100);
          }
        cbi(ADCSRA, ADEN);   // bit 7 of ADCSRA, disable ADC  
        noInterrupts();
        set_sleep_mode(SLEEP_MODE_PWR_DOWN);  
        sleep_enable();  
        sleep_mode();       // all execution should stop here
        while(0==0);        // endless loop (belts and suspenders)
      }
    else PrintOKStr();      // first time through only
  }

//---------------------------------------------------------------------- 
void software_Reset()
  { // Restarts program from beginning but 
    // does not reset the peripherals and registers
    // as we are not doing anything with the the
    // timers or peripherals or registers this
    // should be adequate (will not support updating)

    // Serial.println(prevcmd);
    if ((prevcmd[0]=='!') && (prevcmd[1]=='!'))
      { Serial.println (F("; RESETTING"));
        // give device time to send string
        delay (1000);
        asm volatile ("  jmp 0");
      }
    else PrintOKStr();      // first time through only
  }   

//---------------------------------------------------------------------- 
void SetRawReadMode()
  { // check for "T" or "F", true of false
    if      (cmd[1]=='T') {RtnRawRead = true;  PrintOKStr();}
    else if (cmd[1]=='F') {RtnRawRead = false; PrintOKStr();}
    else PrintNotRecognized();
  }

//---------------------------------------------------------------------- 
void SetCelsiusMode()
  { // check for "T" or "F", true of false
    if      (cmd[1]=='T') {RtnCelsius = true;  PrintOKStr();}
    else if (cmd[1]=='F') {RtnCelsius = false; PrintOKStr();}
    else PrintNotRecognized();
  }

//---------------------------------------------------------------------- 
void SetFahrenheitdMode()
  { // check for "T" or "F", true of false
    if      (cmd[1]=='T') {RtnFahrenh = true;  PrintOKStr();}
    else if (cmd[1]=='F') {RtnFahrenh = false; PrintOKStr();}
    else PrintNotRecognized();
  }

//---------------------------------------------------------------------- 
void SetReportMode()
  { // check for "T" or "F", true of false
    if      (cmd[1]=='T') {ReportMode = true;  PrintOKStr();}
    else if (cmd[1]=='F') {ReportMode = false; PrintOKStr();}
    else PrintNotRecognized();
  }

//---------------------------------------------------------------------- 
void ToggleDebugMode()
  { // toggle Debug mode
    if      (DeBug == true)  DeBug = false;
    else if (DeBug == false) DeBug = true;
    PrintOKStr();
  }

//---------------------------------------------------------------------- 
void ToggleRoundMode()
  { // check for "T" or "F", true of false
    if   (RoundMode) {RoundMode = false; PrintOKStr();}
    else             {RoundMode = true;  PrintOKStr();}
  }
  
//---------------------------------------------------------------------- 
void NewReportTime()
  { // set report Minutes
    if      (cmd[1]=='1') { MinuteTarget =    1; Report_Reset();} 
    else if (cmd[1]=='2') { MinuteTarget =    2; Report_Reset();} 
    else if (cmd[1]=='3') { MinuteTarget =    3; Report_Reset();}
    else if (cmd[1]=='4') { MinuteTarget =    4; Report_Reset();}
    else if (cmd[1]=='5') { MinuteTarget =    5; Report_Reset();}
    else if (cmd[1]=='6') { MinuteTarget =   10; Report_Reset();}
    else if (cmd[1]=='7') { MinuteTarget =   15; Report_Reset();}
    else if (cmd[1]=='8') { MinuteTarget =   20; Report_Reset();}
    else if (cmd[1]=='9') { MinuteTarget =   30; Report_Reset();}
    else if (cmd[1]=='0') { MinuteTarget =   60; Report_Reset();}
    else PrintNotRecognized();                          // not recognized
  }
//---------------------------------------------------------------------- 
void ToggleSensor()
  { // set sensor as True or False
    byte i;
    i=cmd[1]-'0';
    if (i<8) 
      { if (Sensors[i]) Sensors[i]=false; 
        else Sensors[i]=true;
       Serial.print   (F(";   Sensors:\t"));
       PrintSensors();
      }
    
    else PrintNotRecognized();                          // not recognized
  }


//----------------------------------------------------------------------  
void Report_Reset()
  { // this force the current data to be reported 
    // and reset our clock using the new time
    unsigned long SaveMe=SecondsTarget;
    PrintOKStr();
    Serial.println (F("; Report Timing reset"));
    // calculate seconds between report lines
    // SecondsTarget=MinuteTarget*SecondsMinute;
    // we have to "cast" the two word values or we will get a word value for the result
    SecondsTarget=(long(MinuteTarget)*long(SecondsMinute));
    if (SecondsTarget != SaveMe) newflg = newflg |  B00010000;
    
    // reset report parameters
    for (byte i=0; i<8; i++) Accumalator[i] = 0; 
    CycleTime   = 0;
    CycleCount  = 0;
    RptTrigger  = millis() + SecondsTarget;
    RptStartTime= millis();     
  }   

//---------------------------------------------------------------------- 
void NewIdString()
  { // New Location ID String
    // Serial.println("got here: NewIdString");
    // set time out to 5 seconds
    unsigned long timelimit = millis() + (5000);
    boolean timeout=false;
    char c= -1;
    byte n= 0;
    while ((c != 0) && (c != 10) && (c != 9) && (c != 13) && (n<EEidsize) && (timeout==false))
     { delay(10);
       c = Serial.read();
       if (c > 31) IdString[n++]=c;
       // check for timeout
       if (millis()>timelimit) timeout=true;
     }
   while (n<EEidsize) IdString[n++]=0;
   if (timeout) Serial.println (F("; aborted due to timeout"));
   else
    { IdString[EEidsize]=0;  // make certain last charater is null
      newflg = newflg |  B00000001;
      // Serial.println (IdString);
      PrintOKStr();
    }
  DrainCmdTermiantors();
  }
//---------------------------------------------------------------------- 
byte FirstValidSensor()
  { byte i;
  // Return = Terminate a function and return a value from a function
   for (i=0; i<8; i++) {if (Sensors[i]==true) return i;}
   return 0;
  }

//---------------------------------------------------------------------- 
void PrintDegreeOffsetEffect(float NewOffset)
  { // new offset must be in Degrees Fahrenheit 
    boolean SaveRoundMode;
    SaveRoundMode=RoundMode;
    // use first available sensor
    byte UseSensor=FirstValidSensor();
    if (UseSensor!=0)
      { // do it
        while (CycleCount<50) ReadRawTemp();
        DegreeOffset=0;
        RoundMode=false;
        Convert(Accumalator[UseSensor]/CycleCount);
        DegreeOffset=NewOffset;
        Serial.print (F("; Offset(F):\t"));
        Serial.println (DegreeOffset);
        Serial.print (F(";Fahrenheit:\t"));
        Serial.println (Fahrenheit);
        Serial.print (F(";  Adjusted:\t"));
        Fahrenheit=Fahrenheit+NewOffset;
        if (SaveRoundMode) Fahrenheit=nearesthalf(Fahrenheit);
        Serial.println (Fahrenheit);
        //-----------------------------------------------
        Serial.print (F(";   Celsius:\t"));
        Serial.println (Celsius);
        Serial.print (F(";  Adjusted:\t"));
        Celsius=Celsius+(DegreeOffset/1.8000);
        if (SaveRoundMode) Celsius=nearestquater(Celsius);
        Serial.println (Celsius);
        newflg = newflg |  B00000010;
        RoundMode=SaveRoundMode;
        PrintOKStr();
      }
    else Serial.println ("; Valid Sensor not found.");
  }
  
//---------------------------------------------------------------------- 
void ValueNotAccepted()
  { Serial.print("; ");
    Serial.print(cmd);
    Serial.print(" invalid/no input");
  }
//---------------------------------------------------------------------- 
void NewDegreeOffset()
  { // New Degree Offset
    float tempfloat=0;
    delay (2000);
    tempfloat=Serial.parseFloat();
    if (tempfloat!=0)
      { if (abs(tempfloat)<0.010) tempfloat = 0;
        PrintDegreeOffsetEffect(tempfloat);
        newflg = newflg |  B00000010;
      }
    else Serial.println (F("; value not accepted"));    
    DrainCmdTermiantors();
  }

//---------------------------------------------------------------------- 
void CalculateDegreeOffset(float tempfloat)
  { // calculate a new degree offset, TempF is Temperature in degrees Fahrenheit
    // get the current raw reading
    boolean SaveRoundMode;
    // use first available sensor
    byte UseSensor=FirstValidSensor();
    if (UseSensor!=0)
      {
        Serial.println (F("; Calculating new offset ..."));
        while (CycleCount<50) ReadRawTemp();
        // set the current offset to zero so that it
        // does not affect the Conversion
        DegreeOffset=0;
        SaveRoundMode=RoundMode;
        RoundMode=false;
        Convert(Accumalator[UseSensor]/CycleCount);
        RoundMode=SaveRoundMode;
        PrintDegreeOffsetEffect(tempfloat-Fahrenheit);
      }
    else Serial.print (F("No valid sensor found."));
  }  
//---------------------------------------------------------------------- 
void FahrenheitEquals()
 { // sets offset according to current reading and input Fahrenheit
    float tempfloat=0;
    float deltaR;
    word RawReading;
    delay (2000);
    tempfloat=Serial.parseFloat();
    if (tempfloat != 0) CalculateDegreeOffset(tempfloat);
    else ValueNotAccepted();  
    DrainCmdTermiantors();
  }

//---------------------------------------------------------------------- 
void CelsiusEquals()
 { // sets offset according to current reading and input Fahrenheit
    float tempfloat=0;
    float deltaR;
    word RawReading;
    // Serial.println (F("Got Here: CelsiusEquals"));
    delay (2000);
    tempfloat=Serial.parseFloat();
    if (tempfloat != 0) CalculateDegreeOffset((tempfloat*1.8)+32);
    else ValueNotAccepted();  
    DrainCmdTermiantors();
  }

//---------------------------------------------------------------------- 
void NewRefVolt()
  { // New Degree Offset
    float tempfloat=0;
    delay (2000);
    tempfloat=Serial.parseFloat();
    if (tempfloat!=0)
      { // Serial.println(tempfloat,4);
        // Serial.println(RefVoltage,4);
        RefVoltage=tempfloat;
        // Serial.println(RefVoltage,4);
        newflg = newflg |  B00000100;
        PrintOKStr();
      }
    else ValueNotAccepted();
    DrainCmdTermiantors();
  }

//-----------------------------------------------------
void RestoreFromBackup()
  { char TempString[EEwdsize];
    byte i;
    // read the backup copy
    for (i=0; i<EEwdsize; i++) TempString[i]=EEPROM.read(StorageBackup + i);
    // write working copy
    for (i=0; i<EEwdsize; i++) EEPROM.write(StorageWorking +i, TempString[i]);
    newflg=0;
    Read_Calibration_Data();
    Report_Reset();
  }

//---------------------------------------------------------------------- 
void OverwriteBackup()
  { char TempString[EEwdsize];
    byte i;
    // read the working copy
    for (i=0; i<EEwdsize; i++) 
	         TempString[i]=EEPROM.read(StorageWorking + i);
    // write backup copy
    for (i=0; i<EEwdsize; i++) EEPROM.write(StorageBackup +i, TempString[i]);
    PrintOKStr();
  }

//=====================================================================
void TestData1()
 { // These sets were picked for testing
   // so that one set look like the another set.
   char temp[]="Test data,Nano   ";
   //...........1234567890123456
   byte i;
   Serial.println(F("; Load Default Parameters"));
   for (i=0; i< EEidsize; i++) IdString[i]=temp[i];
   // Insert null terminator at end
   IdString[EEidsize]=0;
   DegreeOffset=0;
   RefVoltage  =1.070;
   MinuteTarget=1;
   Sensors[0]=false;
   Sensors[1]=true;
   Sensors[2]=true;
   Sensors[3]=false;
   Sensors[4]=false;
   Sensors[5]=false;
   Sensors[6]=false;
   Sensors[7]=false;
   newflg=0XFF;
   Write_Calibration_Data();
 }


